@@ -59,20 +59,24 @@ namespace pointing
     int debugLevel;
 
     void parseItem(const HIDItem &item);
-    void clearAll();
 
     // Find the report which contains relative X and Y
     bool findCorrectReport();
 
-    // TODO Implement copy constructor which copies
-    // report and curRepInfo correctly
-    HIDReportParser(HIDReportParser const&);
-
   public:
     HIDReportParser();
     HIDReportParser(unsigned char *desc, int size, int debugLevel=0);
+
+    HIDReportParser(const HIDReportParser &);
+    HIDReportParser& operator=(const HIDReportParser &);
+
     ~HIDReportParser();
 
+    /**
+     * @brief Clears all data, so the parser can't parse anymore
+     */
+    void clearDescriptor();
+
     /**
      * @brief setDescriptor Sets the HID descriptor, parses it.
      * @param desc
@@ -92,7 +96,7 @@ namespace pointing
      * @brief getReportLength
      * @return The length of the Mouse Report in bytes deduced from the report descriptor
      */
-    int getReportLength();
+    int getReportLength() const;
 
     /**
      * @brief getDxDy Gives the relative X and Y in mouse points
